            ORG 0
            LJMP   ST1

CTRL        EQU    0E100H
PORTA       EQU    0E101H
PORTB       EQU    0E102H
PORTC       EQU    0E103H
DISBUF      EQU    0E000H
DIGIT0      EQU    DISBUF
DIGIT1      EQU    DISBUF+1
DIGIT2      EQU    DISBUF+2
DIGIT3      EQU    DISBUF+3
DIGIT4      EQU    DISBUF+4
DIGIT5      EQU    DISBUF+5
SER_BUF     EQU    DISBUF+48
NEW_DAT     EQU    7FH           ; Bit addressable

            ORG    23H           ; Serial vector
SR_INT:     CLR    ES
            PUSH   ACC
            PUSH   DPL
            PUSH   DPH
            LCALL  SR_PROC
            POP    DPH
            POP    DPL
            POP    ACC
            SETB   ES
            RETI

            ORG    100H


ST1:        MOV    A,#0FDH       ;Set baud rate 9600
            MOV    TH1,A
            MOV    TL1,A
            MOV    TMOD,#00100000B     ;TIMER1 MODE 1
            SETB   EA            ;Global enable interupt
            CLR    ET1           ;Disable timer1 interupt
            CLR    ES            ;Disable serial port
            SETB   TR1                 ;Set timer1 control
            MOV    SCON,#01010000B     ;Serial mode 1
            SETB   ES            ;Enable interrupt

            MOV    SP,#30H
            LCALL  ROW_DLY
            MOV    DPTR,#CTRL    ; Control port 8155
            MOV    A,#0FH
            MOVX   @DPTR,A
            LCALL  CLR_SCR       ; Clear Screen(Display)
            MOV    DPTR,#SER_BUF ; Set serial port buffer
            MOV    R7,DPH
            MOV    R6,DPL
            MOV    DPTR,#SIGN_ON
            LCALL  MOV_STRG      ; Move messege to display
ST2:        LCALL  DISPLAY
            JNB    NEW_DAT,ST2
            CLR    NEW_DAT
            LCALL  CLR_SCR       ; Clear screen
            MOV    DPTR,#SER_BUF ; New messege
            LCALL  MOV_STRG
            SJMP   ST2

SIGN_ON:    DB     'TEST1',0DH

;*********************************
;*        DISPLAY ROUTINE        *
;*********************************
;  P2:R1    = PortB (clock 74LS374),R1 = low byte address
;  P2:R0    = PortA (Data),R0 = low byte address
;  R3       = Use for :-
;                - Hi-byte = Row count
;                - Lo-byte = Digit count (which 74LS374)
; Note that this routine can interrupt for access
;           thei routine no register destroyed

DISPLAY:    PUSH   B
            PUSH   00
            PUSH   01
            PUSH   03
            PUSH   DPL
            PUSH   DPH
            PUSH   ACC
            MOV    DPTR,#PORTA
            MOV    P2,DPH
            MOV    R0,DPL
            MOV    R1,#LOW PORTB
            MOV    R3,#0          ; Clock the 1st digit
            MOV    A,#0E0H
            MOVX   @R1,A          ; Trig 74LS123 pin 10
DP1:        MOV    A,R3
            SETB   ACC.7          ; Disable 74ls138 (Row)
            MOVX   @R1,A          ; Out port B
            MOV    DPTR,#DISBUF
DP3:        MOV    A,R3
            SETB   ACC.7
            MOVX   @R1,A
DP2:        MOVX   A,@DPTR        ; Read Data from buffer
            INC    DPTR           ; Bump Pointer
            PUSH   DPH            ; Keep pointer
            PUSH   DPL
            MOVX   @R0,A          ; Write data to port A
            INC    R3             ; Shift clock of LS374
            MOV    A,R3
            SETB   ACC.7
            MOVX   @R1,A          ; Clock Precedenc LS374 and
            POP    DPL            ; switch low at CLK of next 74LS374
            POP    DPH
            MOV    A,R3
            ANL    A,#0FH
            CJNE   A,#6,DP2       ; Next LS374

            MOV    A,R3           ; Check last row
            ANL    A,#0F0H
            MOV    R3,A
            SWAP   A
            ANL    A,#0FH
            CJNE   A,#8H,ROW_ACT
            SJMP   OUT_DSP
ROW_ACT:    MOV    A,R3
            CLR    ACC.7          ; Enable LS138
            MOVX   @R1,A          ; Row Active
            LCALL  ROW_DLY

            ;Advance Row

            MOV    A,R3
            SWAP   A
            ANL    A,#0FH         ; 0000XXXX
            INC    A
            SWAP   A
            ORL    A,#0FH         ; XXXX1111
            MOV    B,A            ; Keep in B
            MOV    A,R3
            ORL    A,#0F0H        ; 1111000
            ANL    A,B
            MOV    R3,A           ; Next Row
            LJMP   DP3
OUT_DSP:    POP    ACC
            POP    DPH
            POP    DPL
            POP    03
            POP    01
            POP    00
            POP    B
            RET

;**************************************
;*   Get address of character         *
;*   Input  = ACC = Ascii of character*
;*   Output = DPTR point to 1st byte  *
;**************************************
;
GET_CHR:    PUSH   B
            SUBB   A,#20H      ; Because font start at ASCII 20h
            MOV    B,#08
            MUL    AB
            ADD    A,#LOW FONT   ; Low byte of FONT table
            MOV    DPL,A
            MOV    A,B
            ADDC   A,#HIGH FONT
            MOV    DPH,A
            POP    B
            RET

;*************************************
;*    FILL  FONT TO DISPLAY BUFFER   *
;*  Input   : DPTR Contain addres of *
;*            1st byte of table .    *
;*            R1:R0 = Addres of      *
;*                    display buffer *
;*************************************
FILL_CHR:   PUSH  00H
            PUSH  01H
            MOV   P2,R1
            MOV   R1,#8
FC1:        MOVX  A,@DPTR       ; Get font
            MOVX  @R0,A         ; Fill display buffer
            MOV   A,#6          ;
            ADD   A,R0
            MOV   R0,A
            INC   DPTR
            DJNZ  R1,FC1
            POP   01H
            POP   00H
            RET

;*******************************************
;*            MOVE STRING                  *
;* Function : Get character string pointed *
;*            by DPTR and convert to font  *
;*            and put on display buffer    *
;* Input    : DPTR = Address of string     *
;* Note     : end of string by character 0D*
;*******************************************
;
MOV_STRG:   PUSH   00
            PUSH   01
            PUSH   DPL
            PUSH   DPH
            PUSH   ACC
            MOV    R1,#HIGH DISBUF   ; R0:R1 = display buffer
            MOV    R0,#LOW DISBUF
G1:         MOVX   A,@DPTR           ;Get Ascii
            CJNE   A,#0DH,PROCESS
            SJMP   OUT_MOV
PROCESS:    INC    DPTR
            PUSH   DPL
            PUSH   DPH
            LCALL  GET_CHR
            LCALL  FILL_CHR
            INC    R0                ;Next digit (DISBUF)
            POP    DPH
            POP    DPL
            SJMP   G1                ;Until meet Ascii 1Ah
OUT_MOV:    POP    ACC
            POP    DPH
            POP    DPL
            POP    01
            POP    00
            RET

;**************************************
;*     SERIAL PORT SERVICE ROUTINE    *
;**************************************
;
SR_PROC:    JBC    RI,RX
            CLR    TI
            SJMP   OSR
RX:         MOV    DPH,R7        ; R7:R6 = Serial buffer
            MOV    DPL,R6
            MOV    A,#LOW SER_BUF+5
            CJNE   A,DPL,RX2
            MOV    A,#HIGH SER_BUF
            CJNE   A,DPH,RX2
            MOV    A,#0DH
            MOVX   @DPTR,A
            SJMP   RX5
RX2:        MOV    A,SBUF
            MOV    SBUF,A
            JNB    TI,$
            CLR    TI
            MOVX   @DPTR,A       ; Save data
            CJNE   A,#0DH,RX3
RX5:        SETB   NEW_DAT
            MOV    DPTR,#SER_BUF
            SJMP   RX4
RX3:        INC    DPTR
RX4:        MOV    R7,DPH        ; Keep pointer
            MOV    R6,DPL
OSR:        RET


ROW_DLY:    PUSH   ACC
            PUSH   B
            MOV    B,#2FH
RD1:        DEC    B
            MOV    A,B
            JNZ    RD1
            POP    B
            POP    ACC
            RET

;*******************************
;*        CLEAR SCREEN         *
;*******************************
CLR_SCR:    PUSH   DPL
            PUSH   DPH
            PUSH   0
            MOV    DPTR,#DISBUF
            MOV    R0,#48
F1:         MOV    A,#00
            MOVX   @DPTR,A
            INC    DPTR
            DJNZ   R0,F1
            POP    0
            POP    DPH
            POP    DPL
            RET


FONT:       DB 00H,00H,00H,00H,00H,00H,00H,00H    ; Null
            DB 10H,10H,10H,10H,00H,00H,10H,00H    ; !
            DB 28H,28H,28H,00H,00H,00H,00H,00H    ; "
            DB 28H,28H,7CH,28H,7CH,28H,28H,00H    ; #
            DB 10H,3CH,50H,38H,14H,78H,10H,00H    ; $
            DB 60H,64H,08H,10H,20H,4CH,0CH,00H    ; %
            DB 30H,48H,50H,20H,54H,48H,34H,00H    ; &
            DB 30H,10H,20H,00H,00H,00H,00H,00H    ; '
            DB 08H,10H,20H,20H,20H,10H,08H,00H    ; (
            DB 20H,10H,08H,08H,08H,10H,20H,00H    ; )
            DB 00H,10H,54H,38H,54H,10H,00H,00H    ; *
            DB 00H,10H,10H,7CH,10H,10H,00H,00H    ; +
            DB 00H,00H,00H,00H,30H,10H,20H,00H    ; ,
            DB 00H,00H,00H,7CH,00H,00H,00H,00H    ; -
            DB 00H,00H,00H,00H,00H,30H,30H,00H    ; .
            DB 00H,04H,08H,10H,20H,40H,00H,00H    ; /
            DB 38H,44H,4CH,54H,64H,44H,38H,00H    ; 0
            DB 10H,30H,10H,10H,10H,10H,38H,00H    ; 1
            DB 38H,44H,04H,08H,10H,20H,7CH,00H    ; 2
            DB 7CH,08H,10H,08H,04H,44H,38H,00H    ; 3
            DB 08H,18H,28H,48H,7CH,08H,08H,00H    ; 4
            DB 7CH,40H,78H,04H,04H,44H,38H,00H    ; 5
            DB 18H,20H,40H,78H,44H,44H,38H,00H    ; 6
            DB 7CH,04H,08H,10H,20H,20H,20H,00H    ; 7
            DB 38H,44H,44H,38H,44H,44H,38H,00H    ; 8
            DB 38H,44H,44H,3CH,04H,08H,30H,00H    ; 9
            DB 00H,30H,30H,00H,30H,30H,00H,00H    ; :
            DB 00H,30H,30H,00H,30H,10H,20H,00H    ; ;
            DB 08H,10H,20H,40H,20H,10H,08H,00H    ; <
            DB 00H,00H,7CH,00H,7CH,00H,00H,00H    ; =
            DB 20H,10H,08H,04H,08H,10H,20H,00H    ; >
            DB 38H,44H,04H,08H,10H,00H,10H,00H    ; ?
            DB 38H,44H,04H,34H,54H,54H,38H,00H    ; @
UPPER:      DB 38H,44H,44H,7CH,44H,44H,44H,00H    ; A
            DB 78H,44H,44H,78H,44H,44H,78H,00H    ; B
            DB 38H,44H,40H,40H,40H,44H,38H,00H    ; C
            DB 70H,48H,44H,44H,44H,48H,70H,00H    ; D
            DB 7CH,40H,40H,78H,40H,40H,7CH,00H    ; E
            DB 7CH,40H,40H,78H,40H,40H,40H,00H    ; F
            DB 38H,44H,40H,5CH,44H,44H,3CH,00H    ; G
            DB 44H,44H,44H,7CH,44H,44H,44H,00H    ; H
            DB 38H,10H,10H,10H,10H,10H,38H,00H    ; I
            DB 1CH,08H,08H,08H,08H,48H,30H,00H    ; J
            DB 44H,48H,50H,60H,50H,48H,44H,00H    ; K
            DB 40H,40H,40H,40H,40H,40H,7CH,00H    ; L
            DB 44H,6CH,54H,54H,44H,44H,44H,00H    ; M
            DB 44H,44H,64H,54H,4CH,44H,44H,00H    ; N
            DB 38H,44H,44H,44H,44H,44H,38H,00H    ; O
            DB 78H,44H,44H,78H,40H,40H,40H,00H    ; P
            DB 38H,44H,44H,44H,54H,48H,34H,00H    ; Q
            DB 78H,44H,44H,78H,50H,48H,44H,00H    ; R
            DB 3CH,40H,40H,38H,04H,04H,78H,00H    ; S
            DB 7CH,10H,10H,10H,10H,10H,10H,00H    ; T
            DB 44H,44H,44H,44H,44H,44H,38H,00H    ; U
            DB 44H,44H,44H,44H,44H,28H,10H,00H    ; V
            DB 44H,44H,44H,54H,54H,54H,28H,00H    ; W
            DB 44H,44H,28H,10H,28H,44H,44H,00H    ; X
            DB 44H,44H,44H,28H,10H,10H,10H,00H    ; Y
            DB 7CH,04H,08H,10H,20H,40H,7CH,00H    ; Z
            DB 38H,20H,20H,20H,20H,20H,38H,00H    ; [
            DB 40H,20H,10H,08H,04H,02H,01H,00H    ; \
            DB 38H,08H,08H,08H,08H,08H,38H,00H    ; ]
            DB 10H,28H,44H,00H,00H,00H,00H,00H    ; ^
            DB 00H,00H,00H,00H,00H,00H,7CH,00H    ; _
            DB 20H,10H,08H,00H,00H,00H,00H,00H    ;`
LOWER:      DB 00H,00H,38H,08H,38H,48H,34H,00H    ; a
            DB 40H,40H,58H,64H,44H,44H,78H,00H    ; b
            DB 00H,00H,3CH,40H,40H,40H,3CH,00H    ; c
            DB 04H,04H,34H,4CH,44H,44H,3CH,00H    ; d
            DB 00H,00H,38H,44H,78H,40H,38H,00H    ; e
            DB 18H,24H,20H,70H,20H,20H,20H,00H    ; f
            DB 00H,00H,38H,44H,44H,3CH,04H,38H    ; g
            DB 40H,40H,58H,64H,44H,44H,44H,00H    ; h
            DB 10H,00H,30H,10H,10H,10H,38H,00H    ; i
            DB 00H,08H,00H,08H,08H,28H,10H,00H    ; j
            DB 40H,40H,48H,50H,60H,50H,48H,00H    ; k
            DB 30H,10H,10H,10H,10H,10H,18H,00H    ; l
            DB 00H,00H,68H,54H,54H,54H,44H,00H    ; m
            DB 00H,00H,58H,24H,24H,24H,24H,00H    ; n
            DB 00H,00H,38H,44H,44H,44H,38H,00H    ; o
            DB 00H,00H,70H,48H,48H,70H,40H,40H    ; p
            DB 00H,00H,38H,48H,48H,38H,08H,08H    ; q
            DB 00H,00H,58H,64H,40H,40H,40H,00H    ; r
            DB 00H,00H,38H,40H,38H,04H,78H,00H    ; s
            DB 20H,20H,78H,20H,20H,24H,18H,00H    ; t
            DB 00H,00H,48H,48H,48H,48H,34H,00H    ; u
            DB 00H,00H,44H,44H,28H,28H,10H,00H    ; v
            DB 00H,00H,44H,44H,54H,54H,28H,00H    ; w
            DB 00H,00H,44H,28H,10H,28H,44H,00H    ; x
            DB 00H,00H,48H,48H,48H,38H,08H,70H    ; y
            DB 00H,00H,7CH,08H,10H,20H,7CH,00H    ; z
            DB 08H,10H,10H,20H,10H,10H,08H,00H    ; {
            DB 18H,18H,18H,18H,18H,18H,18H,00H    ; |
            DB 20H,10H,10H,08H,10H,10H,20H,00H    ; }
            DB 00H,00H,00H,32H,4CH,00H,00H,00H    ; ~

            END


